"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrintSubmissionResults = exports.PrintTestDetails = exports.ReadProblemTestCases = exports.ReadProblemSolution = exports.PrintUserSummary = exports.RequestPassword = exports.OpenLoginBrowser = exports.VerifyPassword = exports.HashPassword = exports.PrintQuestionSummary = exports.PrintProblemsSummary = exports.PrintUsedFilters = exports.CreateQuestionInstance = exports.TimestampToDate = exports.JustifyString = exports.Transpose = exports.ArraySum = exports.FormatString = exports.GetAllProblemsCount = exports.FormatCookies = exports.FormatShortSubmissionDetails = exports.FormatUserData = exports.FormatProblemsData = exports.FormatQuestionData = exports.GetExistingProblems = void 0;
const promises_1 = require("fs/promises");
const path_1 = require("path");
const crypto_1 = require("crypto");
const pprint_1 = require("./pprint");
const turndown_1 = __importDefault(require("turndown"));
const constants_1 = __importDefault(require("./constants"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const prompt_1 = __importDefault(require("prompt"));
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const lc = __importStar(require("./leetcode"));
const GetExistingProblems = async (problem_folder) => {
    try {
        var result = [];
        if (!fs.existsSync(problem_folder))
            return [];
        const files = await (0, promises_1.readdir)(problem_folder, { withFileTypes: true });
        result = files.map((value) => {
            return Number.parseInt(value.name.split("-")[0]);
        });
        return result;
    }
    catch (error) {
        console.error("Error reading folder: ", error);
        return [];
    }
};
exports.GetExistingProblems = GetExistingProblems;
const FormatQuestionData = (data) => ({
    link: `https://leetcode.com/problems/` + data.question.titleSlug,
    question: {
        isPaidOnly: data.question.isPaidOnly,
        questionId: data.question.questionId,
        questionFrontendId: data.question.questionFrontendId,
        title: data.question.title,
        titleSlug: data.question.titleSlug,
        difficulty: data.question.difficulty,
        content: data.question.content,
        exampleTestcaseList: data.question.exampleTestcaseList,
        topicTags: data.question.topicTags,
        companyTagStats: data.question.companyTagStats,
        hints: data.question.hints,
        solution: data.question.solution,
        codeSnippets: data.question.codeSnippets,
        status: data.question.status
    },
});
exports.FormatQuestionData = FormatQuestionData;
const FormatProblemsData = (data) => ({
    totalQuestions: data.problemsetQuestionList.total,
    count: data.problemsetQuestionList.questions.length,
    problemsetQuestionList: data.problemsetQuestionList.questions,
});
exports.FormatProblemsData = FormatProblemsData;
const FormatUserData = (user, lstats, subList, acSubList) => ({
    link: `https://leetcode.com/u/${user.matchedUser.username}`,
    profile: {
        username: user.matchedUser.username,
        realName: user.matchedUser.profile.realName,
        aboutMe: user.matchedUser.profile.aboutMe,
        reputation: user.matchedUser.profile.reputation,
        ranking: user.matchedUser.profile.ranking,
        githubUrl: user.matchedUser.githubUrl,
        twitterUrl: user.matchedUser.twitterUrl,
        linkedinUrl: user.matchedUser.linkedinUrl,
        websites: user.matchedUser.profile.websites,
        skillTags: user.matchedUser.profile.skillTags
    },
    submitStats: user.matchedUser.submitStats,
    langStats: lstats.matchedUser.languageProblemCount,
    subList: subList,
    acSubList: acSubList
});
exports.FormatUserData = FormatUserData;
const FormatShortSubmissionDetails = (details) => ({
    runtimeDisplay: details.submissionDetails?.runtimeDisplay,
    memoryDisplay: details.submissionDetails?.memoryDisplay,
    question: details.submissionDetails?.question,
    lang: details.submissionDetails?.lang
});
exports.FormatShortSubmissionDetails = FormatShortSubmissionDetails;
const FormatCookies = (cookies) => {
    if (!cookies)
        return undefined;
    const cookie_s = `LEETCODE_SESSION=${cookies.LEETCODE_SESSION}; ` +
        `csrftoken=${cookies.csrftoken};`;
    return { "Cookie": cookie_s };
};
exports.FormatCookies = FormatCookies;
const GetAllProblemsCount = async () => ((await Promise.all(constants_1.default.DIFFICULTIES.map(async (x) => {
    const nproblems = await lc.FetchNumberOfProblems(x);
    return { [x]: nproblems ?? 0 };
}))).reduce((p, c) => ({ ...p, ...c })));
exports.GetAllProblemsCount = GetAllProblemsCount;
const SaveHtmlToMarkdown = (path, content) => {
    // Initialize turndown service and convert
    const markdown = (new turndown_1.default()).turndown(content);
    fs.writeFileSync(path, markdown);
};
const FormatString = (template, ...args) => {
    return template.replace(/{(\d+)}/g, (_, index) => args[index] || "");
};
exports.FormatString = FormatString;
const ArraySum = (...values) => values.reduce((x, y) => x + y);
exports.ArraySum = ArraySum;
const Transpose = (matrix) => {
    return matrix[0].map((_, colIndex) => matrix.map(row => row[colIndex]));
};
exports.Transpose = Transpose;
const CenterString = (str, width) => {
    // Calculate the total padding required
    const padding = width - str.length;
    // If the string is already wider than the desired width, return the original string
    if (padding <= 0) {
        return str;
    }
    // Calculate left and right padding
    const leftPadding = Math.floor(padding / 2);
    const rightPadding = Math.ceil(padding / 2);
    // Return the centered string
    return ' '.repeat(leftPadding) + str + ' '.repeat(rightPadding);
};
const JustifyString = (str, width, dir) => {
    if (dir == 0)
        return CenterString(str, width); // Center the string
    const remaining_size = (width - str.length) > 0;
    if (dir == -1)
        return str + ((remaining_size) ? ' '.repeat(width - str.length) : '');
    if (dir == 1)
        return ((remaining_size) ? ' '.repeat(width - str.length) : '') + str;
    return str;
};
exports.JustifyString = JustifyString;
const TimestampToDate = (timestamp) => ((new Date(timestamp * 100)).toLocaleString());
exports.TimestampToDate = TimestampToDate;
const CreateQuestionInstance = (question, output) => {
    // Check if the input data is null
    if (question === null) {
        console.error(chalk_1.default.redBright("[ERROR] Impossible to create problem instance. Input is null."));
        return;
    }
    const question_root_folder = output || constants_1.default.OUTPATH;
    // Create also the root folder if it does not exists
    if (!fs.existsSync(question_root_folder)) {
        if (fs.mkdirSync(question_root_folder, { recursive: true }) === undefined) {
            console.error(chalk_1.default.redBright(`[ERROR] Impossible to create root folder: ${question_root_folder}`));
            return;
        }
    }
    // Create the problem folder name and file name
    const question_name = question.question.titleSlug;
    const question_id = question.question.questionFrontendId;
    const question_full_name = question_id.toString().concat("-", question_name);
    const question_folder = (0, path_1.join)(question_root_folder, question_full_name);
    const question_src = (0, path_1.join)(question_folder, "solution.py");
    const question_tests = (0, path_1.join)(question_folder, "tests.txt");
    const question_readme = (0, path_1.join)(question_folder, "README.md");
    const question_html = (0, path_1.join)(question_folder, "index.html");
    const question_tags = question.question.topicTags
        .map((value) => value.name)
        .reduce((prev, curr) => {
        return prev + ", " + curr;
    });
    // Python header with some information about the problem
    const python_header = "\"\"\"\n" +
        "QUESTION TITLE: " + question.question.title + "\n" +
        "QUESTION LINK: " + question.link + "\n" +
        "QUESTION TAGS: " + question_tags + "\n" +
        "QUESTION LEVEL: " + question.question.difficulty + "\n" +
        "\"\"\"";
    // Build the source code that will be written into the python file
    const source_code = "{0}\n\nfrom typing import *\n\n# SOLUTION STARTS\n{1}\n";
    // Take the correct code snippet (Python3)
    const code_snippet = question.question.codeSnippets
        .filter((value) => value.lang === 'Python3')
        .map((value) => value.code)[0] + "\n        ...";
    const full_source_code = (0, exports.FormatString)(source_code, python_header, code_snippet);
    const tests = question.question.exampleTestcaseList
        .map((x) => x.replace('\n', ',,'))
        .join('\n');
    // First we need to create the question folder (it should not exist)
    fs.mkdir(question_folder, 0o777, (err) => {
        if (err)
            console.error(chalk_1.default.redBright(`[ERROR] Operation not permitted: ${err}`));
    });
    // Write the README file using the conversion operation
    SaveHtmlToMarkdown(question_readme, question.question.content);
    // Save the source code content into the source file
    fs.writeFileSync(question_src, full_source_code);
    fs.writeFileSync(question_html, question.question.content);
    fs.writeFileSync(question_tests, tests);
    // Log the result
    console.log(chalk_1.default.greenBright(`[INFO] Result written in folder: ${question_folder}`));
};
exports.CreateQuestionInstance = CreateQuestionInstance;
const PrintUsedFilters = (vars) => {
    // Print the used filters
    const filter_table = new pprint_1.TablePrinter(undefined, undefined, [
        { size: 23, just: -1 },
        { size: 15, style: chalk_1.default.blueBright, just: -1 }
    ]);
    constants_1.default.APP.LIST_QUERY_VARIABLES.forEach((x) => {
        const var_type = vars[x].type; // Take the corresponding type
        const var_value = (var_type == "n") ? vars[x].value.toString() : vars[x].value;
        filter_table.pushRow(`Used Filter ${x}:`, var_value);
    });
    filter_table.showLine = false;
    console.log(filter_table.toString());
};
exports.PrintUsedFilters = PrintUsedFilters;
const PrintProblemsSummary = async (problems, vars) => {
    console.log("Total Problems   :", problems.totalQuestions);
    console.log("Fetched Problems :", problems.count);
    // Get the list of all problems already downloaded
    const downloaded_problems = await (0, exports.GetExistingProblems)(vars['FOLDER'].value);
    // Create the table for the problem list
    const list_table = new pprint_1.TablePrinter(undefined, ['STATUS', 'IDX', 'ID', 'DIFFICULTY', 'TITLE', 'TAGS'], [
        { size: 8, just: 1 }, // Status column
        { size: 5, style: chalk_1.default.yellowBright }, // Local index column
        { size: 5, style: chalk_1.default.yellowBright }, // Problem ID column
        { size: 10, style: constants_1.default.APP.DIFFICULTY_STYLE }, // The difficulty column
        { size: 65, style: chalk_1.default.italic, just: -1 }, // The title column
        { size: 81, style: chalk_1.default.gray, just: -1 } // The tags column
    ]);
    problems.problemsetQuestionList.forEach((value, idx) => {
        const tags = value.topicTags
            .map((value) => value.name)
            .reduce((p, c) => p + ", " + c);
        let status = '';
        // Check if the current question is already downloaded
        if (downloaded_problems.includes(value.questionFrontendId)) {
            status += constants_1.default.APP.EMOJIS.DOWNLOADED;
        }
        // Now check the status of the question
        if (value.status && value.status === "ac") {
            status += constants_1.default.APP.EMOJIS.CHECK;
        }
        else {
            status += constants_1.default.APP.EMOJIS.WRONG;
        }
        // Check if it is premium question
        if (value.isPaidOnly) {
            status += constants_1.default.APP.EMOJIS.NOT_FREE;
        }
        else {
            status += constants_1.default.APP.EMOJIS.FREE;
        }
        // Check if the video solution is available
        if (value.hasVideoSolution) {
            status += constants_1.default.APP.EMOJIS.HAS_VIDEO;
        }
        list_table.pushRow(status, idx, value.questionFrontendId, value.difficulty, value.title, tags);
    });
    list_table.showLine = false;
    console.log(list_table.toString());
};
exports.PrintProblemsSummary = PrintProblemsSummary;
const PrintQuestionSummary = (question) => {
    console.log((0, exports.FormatString)("Last Question: { ID={0}, TitleSlug={1}, Link={2} }", question.question.questionFrontendId, question.question.titleSlug, question.link));
};
exports.PrintQuestionSummary = PrintQuestionSummary;
const HashPassword = (password, salt) => {
    const iterations = constants_1.default.CRYPTO.ITERATIONS;
    const key_length = constants_1.default.CRYPTO.KEY_LENGTH;
    const digest = constants_1.default.CRYPTO.DIGEST;
    const hash = (0, crypto_1.pbkdf2Sync)(password, salt, iterations, key_length, digest);
    return hash.toString("hex");
};
exports.HashPassword = HashPassword;
const VerifyPassword = (password, salt, original_hash) => {
    return (0, exports.HashPassword)(password, salt) === original_hash;
};
exports.VerifyPassword = VerifyPassword;
const OpenLoginBrowser = async (res_cb, req_cb) => {
    const browser = await puppeteer_1.default.launch({ headless: false, args: ['--window-size=1080,1024'] });
    const page = await browser.newPage();
    page.setRequestInterception(true);
    page.on('response', res_cb);
    page.on('request', req_cb);
    await page.goto(constants_1.default.SITES.LOGIN_PAGE.URL, { waitUntil: 'load' });
    page.setViewport({ width: 1080, height: 1024 });
    // Wait until the browser is not closed
    await new Promise((resolve) => {
        browser.on('disconnected', resolve);
    });
};
exports.OpenLoginBrowser = OpenLoginBrowser;
const RequestPassword = async (credentials) => {
    let attemps = constants_1.default.CRYPTO.AUTH_ATTEMPTS;
    prompt_1.default.message = '';
    prompt_1.default.delimiter = '';
    while (attemps > 0) {
        try {
            prompt_1.default.start({ noHandleSIGINT: true });
            const { password } = await prompt_1.default.get(constants_1.default.PROMPT.VALIDATION_SCHEMA);
            const validation_result = (0, exports.VerifyPassword)(password, credentials.salt, credentials.password);
            if (validation_result)
                return true;
        }
        catch (error) {
        }
        console.error(chalk_1.default.redBright("[ERROR] Wrong Password. Retry."));
        attemps--;
    }
    return false;
};
exports.RequestPassword = RequestPassword;
const PrintSubmissionStats = (submitStats) => {
    const total_subs = submitStats?.totalSubmissionNum;
    const acc_subs = submitStats?.acSubmissionNum;
    if (total_subs !== undefined && total_subs.length > 0) {
        const difficulties = total_subs.map((x) => x.difficulty);
        const columns = ['Difficulty', ...difficulties];
        const columns_props = Array(difficulties.length).fill({ size: 10, style: chalk_1.default.yellowBright });
        const props = [{ size: 12 }, ...columns_props];
        const tsubs_table = new pprint_1.TablePrinter("Total Submissions Count", columns, props);
        const counts = ['Count', ...total_subs.map((x) => x.count)];
        const subnum = ['Submissions', ...total_subs.map((x) => x.submissions)];
        const accepted = ['Accepted', ...acc_subs.map((x) => x.submissions)];
        tsubs_table.pushRow(...counts);
        tsubs_table.pushRow(...subnum);
        tsubs_table.pushRow(...accepted);
        console.log();
        console.log(tsubs_table.toString());
        console.log();
    }
};
const PrintShortSubmissionsDetails = (submissions) => {
    // For an optimal columns size, we should take the maximum between all sizes
    const sublist = submissions.submissionList;
    if (sublist.length < 1)
        return;
    const columns = constants_1.default.APP.SHORT_SUBMISSION.COLS;
    const colsizes = Array.from({ length: columns.length }, (_, i) => columns[i].length);
    sublist.forEach((x) => {
        const question_id_len = x.question?.questionId.length ?? 0;
        const lang_len = x.lang?.verboseName.length ?? 0;
        const runtime_len = x.runtimeDisplay?.length ?? 0;
        const memory_len = x.memoryDisplay?.length ?? 0;
        const timestamp = (0, exports.TimestampToDate)(Number.parseInt(x.timestamp));
        colsizes[0] = Math.max(colsizes[0], x.id.length);
        colsizes[1] = Math.max(colsizes[1], x.title.length);
        colsizes[2] = Math.max(colsizes[2], question_id_len);
        colsizes[3] = Math.max(colsizes[3], timestamp.length);
        colsizes[4] = Math.max(colsizes[4], lang_len);
        colsizes[5] = Math.max(colsizes[5], runtime_len);
        colsizes[6] = Math.max(colsizes[6], memory_len);
    });
    const properties = colsizes.map((x, i) => ({
        size: x,
        style: constants_1.default.APP.SHORT_SUBMISSION.STYLES[i],
        just: constants_1.default.APP.SHORT_SUBMISSION.JUST[i]
    }));
    const table = new pprint_1.TablePrinter("Recent Accepted Submissions", columns, properties);
    table.showLine = false;
    sublist.forEach((x) => {
        const question_id = x.question?.questionId ?? "null";
        const lang = x.lang?.verboseName ?? "null";
        const runtime = x.runtimeDisplay ?? "null";
        const memory = x.memoryDisplay ?? "null";
        const timestamp = (0, exports.TimestampToDate)(Number.parseInt(x.timestamp));
        table.pushRow(x.id, x.title, question_id, timestamp, lang, runtime, memory);
    });
    console.log();
    console.log(table.toString());
    console.log();
};
const PrintUserSummary = (user) => {
    console.log(`${chalk_1.default.bold("Username")}   : ${chalk_1.default.blueBright(user.profile?.username)}`);
    console.log(`${chalk_1.default.bold("Real Name")}  : ${chalk_1.default.gray(user.profile?.realName)}`);
    console.log(`${chalk_1.default.bold("Profile")}    : ${chalk_1.default.underline(user.link)}`);
    if (user.profile?.aboutMe !== undefined && user.profile?.aboutMe !== "") {
        const about_box = new pprint_1.RectangleBox(80, 0, chalk_1.default.italic);
        about_box.setTitle("About Me", 4);
        about_box.setContent(user.profile?.aboutMe, { upad: 1, dpad: 1, lpad: 2, rpad: 2 });
        console.log();
        console.log(about_box.toString());
        console.log();
    }
    console.log(`${chalk_1.default.bold("Reputation")} :`, user.profile?.reputation);
    console.log(`${chalk_1.default.bold("Ranking")}    :`, user.profile?.ranking);
    console.log(`${chalk_1.default.bold("Github")}     :`, chalk_1.default.underline(user.profile?.githubUrl));
    console.log(`${chalk_1.default.bold("Twitter")}    :`, chalk_1.default.underline(user.profile?.twitterUrl));
    console.log(`${chalk_1.default.bold("Linkedin")}   :`, chalk_1.default.underline(user.profile?.linkedinUrl));
    PrintSubmissionStats(user.submitStats);
    PrintShortSubmissionsDetails(user.acSubList);
};
exports.PrintUserSummary = PrintUserSummary;
const ReadProblemSolution = (folder, problem_id, problem_title) => {
    if (!fs.existsSync(folder)) {
        console.error(chalk_1.default.redBright(`[ERROR] ${folder} does not exists.`));
        return null;
    }
    try {
        const problem_file = (0, path_1.join)(folder, `${problem_id}-${problem_title}`, 'solution.py');
        const content = fs.readFileSync(problem_file, { encoding: 'utf-8' });
        // Split the string using the line terminator character
        const lines = content.split('\n');
        const final_lines = [];
        for (const line of lines) {
            if (line.includes('TESTS'))
                break;
            final_lines.push(line);
        }
        return final_lines.join('\n');
    }
    catch (error) {
        console.error(chalk_1.default.redBright("[ERROR]", error));
        return null;
    }
};
exports.ReadProblemSolution = ReadProblemSolution;
const ReadProblemTestCases = (folder, problem_id, problem_title) => {
    if (!fs.existsSync(folder)) {
        console.error(chalk_1.default.redBright(`[ERROR] ${folder} does not exists.`));
        return null;
    }
    try {
        const test_file = (0, path_1.join)(folder, `${problem_id}-${problem_title}`, 'tests.txt');
        const content = fs.readFileSync(test_file, { encoding: 'utf-8' });
        const lines = content.split('\n');
        return lines.map((x) => x.replace(',,', '\n'));
    }
    catch (error) {
        console.error(chalk_1.default.redBright("[ERROR]", error));
        return null;
    }
};
exports.ReadProblemTestCases = ReadProblemTestCases;
const PrintTestDetails = (problem, result) => {
    console.log();
    console.log(`Test results for problem: ${chalk_1.default.bold(problem.question.title)}`);
    console.log();
    console.log(`${chalk_1.default.italic("Runtime")}     :`, result.status_runtime);
    console.log(`${chalk_1.default.italic("Memory")}      :`, result.status_memory);
    console.log(`${chalk_1.default.italic("Finish Time")} :`, (0, exports.TimestampToDate)(result.task_finish_time));
    console.log(`${chalk_1.default.italic("Elased Time")} : ${result.elapsed_time / 1e3} [sec]`);
    console.log(`${chalk_1.default.italic("Language")}    :`, chalk_1.default.blueBright(result.pretty_lang));
    console.log();
    if (result.status_msg === "Runtime Error") {
        console.error(chalk_1.default.redBright("[ERROR] Runtime Error:", result.runtime_error));
        console.log();
        return;
    }
    const StatusDisplay = (x) => {
        if (x.includes('OK'))
            return chalk_1.default.greenBright(x);
        return chalk_1.default.redBright(x);
    };
    const table = new pprint_1.TablePrinter(undefined, ['OUTPUT', 'EXPECTED OUTPUT', 'TEST', 'STATUS'], [
        { size: 6, style: chalk_1.default.yellowBright },
        { size: 15, style: chalk_1.default.yellowBright },
        { size: 40, style: chalk_1.default.gray },
        { size: 6, style: StatusDisplay }
    ]);
    for (let i = 0; i < result.total_testcases; i++) {
        const output = result.code_answer[i];
        const expected = result.expected_code_answer[i];
        const test = result.test_cases[i].replace('\n', ', ');
        const status = (output === expected) ? 'OK' : 'NO';
        table.pushRow(output, expected, test, status);
    }
    console.log(table.toString());
    console.log();
};
exports.PrintTestDetails = PrintTestDetails;
const PrintSubmissionResults = (problem, submission) => {
    const failed = submission.total_correct < submission.total_testcases;
    const status = (failed) ? chalk_1.default.redBright("FAILED") : chalk_1.default.greenBright("SUCCESS");
    console.log();
    console.log(`Submission results for problem: ${chalk_1.default.bold(problem.question.title)}`);
    console.log();
    console.log(`${chalk_1.default.italic("Final Status")}  :`, status);
    console.log(`${chalk_1.default.italic("Submission Id")} :`, chalk_1.default.yellowBright(submission.submission_id));
    console.log(`${chalk_1.default.italic("Runtime")}       :`, submission.status_runtime);
    console.log(`${chalk_1.default.italic("Memory")}        :`, submission.status_memory);
    console.log(`${chalk_1.default.italic("Finish Time")}   :`, (0, exports.TimestampToDate)(submission.task_finish_time));
    console.log(`${chalk_1.default.italic("Elased Time")}   : ${submission.elapsed_time / 1e3} [sec]`);
    console.log(`${chalk_1.default.italic("Language")}      :`, chalk_1.default.blueBright(submission.pretty_lang));
    console.log();
    if (submission.status_msg === "Runtime Error") {
        console.error(chalk_1.default.redBright("[ERROR] Runtime Error:", submission.runtime_error));
        console.log();
        return;
    }
    const table = new pprint_1.TablePrinter("Last Test Case", ['OUTPUT', 'EXPECTED', 'TEST CASE'], [
        { size: 20, style: chalk_1.default.yellowBright },
        { size: 20, style: chalk_1.default.yellowBright },
        { size: 40, style: chalk_1.default.gray }
    ]);
    table.showLine = false;
    table.pushRow(submission.code_output, submission.expected_output, submission.last_testcase);
    console.log(table.toString());
    console.log();
    const ShowsPerc = (x) => {
        if (x < 50)
            return chalk_1.default.redBright(x);
        if (x < 100)
            return chalk_1.default.rgb(255, 165, 0)(x);
        return chalk_1.default.greenBright(x);
    };
    const correct = submission.total_correct;
    const total = submission.total_testcases;
    const perc = correct / total * 100;
    console.log(`${chalk_1.default.italic("Result")} : ${correct}/${total} (${ShowsPerc(perc)} %)`);
    console.log();
};
exports.PrintSubmissionResults = PrintSubmissionResults;
